module Blackfire
  module Query
    class << self
      attr_accessor :expires, :user_id, :agent_ids, :profile_slot, :collab_token,
                    :signature, :data, :aggreg_samples, :flag_composer, :flag_cpu,
                    :flag_fn_args, :flag_memory, :flag_nw, :flag_pdo, :flag_sessions,
                    :flag_timespan, :flag_yml, :no_anon, :no_pruning, :timespan_threshold,
                    :timespan_threshold_max, :timespan_threshold_min, :timespan_threshold_ratio,
                    :probed_features, :profile_title

      def parse(query = nil)
        cleanup

        @data = query || ENV["BLACKFIRE_QUERY"]
        return if @data.nil?

        items = @data.split("&").map { |item| item.split("=") }
        populate items.to_h
      end

      private

      def populate(items_hash)
        @expires = items_hash["expires".freeze]
        @user_id = items_hash["userId".freeze]
        @agent_ids = items_hash["agentIds".freeze]
        @profile_slot = items_hash["profileSlot".freeze]
        @collab_token = items_hash["collabToken".freeze]
        @signature = items_hash["signature".freeze]
        @aggreg_samples = items_hash["aggreg_samples".freeze]
        @flag_composer = items_hash["flag_composer".freeze]
        @flag_cpu = items_hash["flag_cpu".freeze]
        @flag_fn_args = items_hash["flag_fn_args".freeze]
        @flag_memory = items_hash["flag_memory".freeze]
        @flag_nw = items_hash["flag_nw".freeze]
        @flag_pdo = items_hash["flag_pdo".freeze]
        @flag_sessions = items_hash["flag_sessions".freeze]
        @flag_timespan = items_hash["flag_timespan".freeze]
        @flag_yml = items_hash["flag_yml".freeze]
        @no_anon = items_hash["no_anon".freeze]
        @no_pruning = items_hash["no_pruning".freeze]
        @timespan_threshold = items_hash["timespan_threshold".freeze]
        @timespan_threshold_max = items_hash["timespan_threshold_max".freeze]
        @timespan_threshold_min = items_hash["timespan_threshold_min".freeze]
        @timespan_threshold_ratio = items_hash["timespan_threshold_ratio".freeze]
        @profile_title = items_hash["profile_title".freeze].nil? ? ($PROGRAM_NAME.split("/")[-1]).to_s.freeze : items_hash["profile_title".freeze]
        @probed_features = build_probed_features
      end

      def build_probed_features
        "aggreg_samples=#{@aggreg_samples}&flag_composer=#{@flag_composer}&flag_cpu=#{@flag_cpu}" \
          "&flag_fn_args=#{@flag_fn_args}&flag_memory=#{@flag_memory}&flag_nw=#{@flag_nw}" \
          "&flag_pdo=#{@flag_pdo}&flag_sessions=#{@flag_sessions}&flag_timespan=#{@flag_timespan}" \
          "&flag_yml=#{@flag_yml}&no_anon=#{@no_anon}&no_pruning=#{@no_pruning}" \
          "&timespan_threshold=#{@timespan_threshold}&timespan_threshold_max=#{@timespan_threshold_max}&" \
          "timespan_threshold_min=#{@timespan_threshold_min}&timespan_threshold_ratio=#{@timespan_threshold_ratio}"
      end

      def cleanup
        @expires = nil
        @user_id = nil
        @agent_ids = nil
        @profile_slot = nil
        @collab_token = nil
        @signature = nil
        @aggreg_samples = nil
        @flag_composer = nil
        @flag_cpu = nil
        @flag_fn_args = nil
        @flag_memory = nil
        @flag_nw = nil
        @flag_pdo = nil
        @flag_sessions = nil
        @flag_timespan = nil
        @flag_yml = nil
        @no_anon = nil
        @no_pruning = nil
        @timespan_threshold = nil
        @timespan_threshold_max = nil
        @timespan_threshold_min = nil
        @timespan_threshold_ratio = nil
      end
    end
  end
end
