# rubocop:disable Style/ClassVars

module Rack
  class BlackfireRack
    def initialize(app, _options = {})
      @app = app
      @@current_path = ""
      @@current_header = ""
    end

    def call(env)
      logger = ::Blackfire::Logger.build

      begin
        request = Rack::Request.new(env)

        if request.env.include?("HTTP_X_BLACKFIRE_QUERY") && (should_profile? request.path)
          if request.path != @@current_path
            @@current_path = request.path
          end

          ::Blackfire.enable(request.env["HTTP_X_BLACKFIRE_QUERY"])
        end
      rescue => e
        logger.error ([e.message]+e.backtrace).join($/)
      end

      # call the application
      result = @app.call(env)

      begin
        ::Blackfire.end
        result[1]["X-Blackfire-Response"] = @@current_header unless @@current_header.empty?
      rescue => e
        logger.error ([e.message]+e.backtrace).join($/)
      end

      result
    end

    def self.define_header(header)
      @@current_header = if header.include?("Blackfire-Response:")
                           ret = if header.include?("\n")
                                   header.split("\n")[0]
                                 else
                                   head
                                 end
                           ret.split("Blackfire-Response:")[1]
                         else
                           ""
                         end
    end

    private

    def should_profile?(path)
      return false if paths_match?(path, [%r{^/assets}, /\.(css|js|png|jpeg|jpg|gif)$/])

      true
    end

    def paths_match?(path, paths)
      paths.any? { |skip_path| skip_path =~ path }
    end
  end
end
# rubocop:enable Style/ClassVars
